mod patterns;
mod bluetooth;

use patterns::Strip;
use bluer::gatt::remote::Characteristic;
use tokio::sync::watch;

const SERVICE_UUID: bluer::Uuid = bluer::Uuid::from_u128(0xadaf0900c33242a893bd25e905756cb8);
const PIXEL_DATA_UUID: bluer::Uuid = bluer::Uuid::from_u128(0xadaf0903c33242a893bd25e905756cb8);
const STRIP_SIZE: usize = 25;
const BASE_STRIP_DATA : [u8; 3] = [0x00, 0x00, 0x01];


#[tokio::main(flavor = "current_thread")]
async fn main() -> bluer::Result<()> {

    let adapter = bluetooth::create_session().await?;

    let devices = bluetooth::scan_devices(adapter, SERVICE_UUID).await;

    let (tx, rx) = watch::channel(Strip::<STRIP_SIZE>::default());

    for device in devices {
        bluetooth::connect(&device, 3).await?;
        let char = bluetooth::get_char(&device, SERVICE_UUID, PIXEL_DATA_UUID).await?;
        if let Some(char) = char {
            let mut rx = rx.clone();
            tokio::spawn(async move {
                while rx.changed().await.is_ok() {
                    let strip = *rx.borrow();
                    write_strip(&strip, &char).await;
                }
            }).await;
        }

        println!("{:?}", device);
    }



    Ok(())
}



// async fn send_seq(char: &Characteristic) -> bluer::Result<()> {
//     println!("  Characteristic flags : {:?}, ", char.flags().await?);
//
//     let mut strip_red : Strip<10> = Strip::default();
//     strip_red.strip.fill(PixelColor { red: 255, green: 0, blue: 0});
//
//     let mut strip_green: Strip<10> = Strip::default();
//     strip_green.strip.fill(PixelColor { red: 0, green: 255, blue: 0});
//
//     let pattern = RainbowPattern::<STRIP_SIZE> { current_iteration: 0,};
//
//
//     for strip in pattern {
//         write_strip(strip, char).await?;
//         sleep(Duration::from_millis(50)).await;
//     }
//
//     Ok(())
// }

pub async fn write_strip<const N: usize>(data: &Strip<N>, char: &Characteristic) -> bluer::Result<()> {
    let frame = [BASE_STRIP_DATA.to_vec(), data.to_array()].concat();
    print!("{:?}",frame);
    char.write(&*frame).await?;
    Ok(())
}



// async fn find_neopixel_service(device: &Device) -> bluer::Result<Option<Characteristic>> {
//     let addr = device.address();
//     let uuids = device.uuids().await?.unwrap_or_default();
//
//     if uuids.contains(&SERVICE_UUID) {
//         println!("service neopixel found for device {}", &addr);
//         match connect(device, 3).await {
//             Ok(()) => {
//                 println!("successefully connected");
//             }
//             Err(err) => {
//                 println!("unable to connect, trying new device, {}", err);
//                 return Err(err)
//             }
//         }
//         for service in device.services().await? {
//             if SERVICE_UUID == service.uuid().await? {
//                 for char in service.characteristics().await? {
//                     println!("uuid : {}", &char.uuid().await?);
//                     if PIXEL_DATA_UUID == char.uuid().await? {
//
//                         return Ok(Some(char));
//                     }
//
//                 }
//             }
//         }
//
//     }
//
//     Ok(None)
// }